@@ -35,14 +35,18 @@ var ENVIRONMENT_IS_SHELL = typeof read === 'function';
 #if ASSERTIONS
 #if !ENVIRONMENT_MAY_BE_NODE && !ENVIRONMENT_MAY_BE_SHELL
 var ENVIRONMENT_IS_WEB = true
-#else
-#if ENVIRONMENT && ENVIRONMENT.indexOf(',') < 0
+#elif ENVIRONMENT && !ENVIRONMENT.includes(',')
 var ENVIRONMENT_IS_WEB = {{{ ENVIRONMENT === 'web' }}};
-#else
+#elif ENVIRONMENT_MAY_BE_SHELL && ENVIRONMENT_IS_NODE
 var ENVIRONMENT_IS_WEB = !ENVIRONMENT_IS_NODE && !ENVIRONMENT_IS_SHELL;
+#elif ENVIRONMENT_MAY_BE_SHELL
+var ENVIRONMENT_IS_WEB = !ENVIRONMENT_IS_SHELL;
+#else
+var ENVIRONMENT_IS_WEB = !ENVIRONMENT_IS_NODE;
+#else
+
 #endif
-#endif
-#endif
+#endif // ASSERTIONS
 
 #if ASSERTIONS && ENVIRONMENT_MAY_BE_NODE && ENVIRONMENT_MAY_BE_SHELL
 if (ENVIRONMENT_IS_NODE && ENVIRONMENT_IS_SHELL) {
