@@ -3,6 +3,7 @@
 #include <string.h>
 #include <thread>
 #include <emscripten.h>
+#include <sanitizer/lsan_interface.h>
 
 void *global_ptr;
 thread_local void *tls_ptr;
@@ -22,10 +23,6 @@ void g(void) {
   while (1);
 }
 
-namespace __lsan {
-  void DoLeakCheck();
-}
-
 int main(int argc, char **argv) {
   std::thread t(g);
   t.detach();
@@ -39,6 +36,6 @@ int main(int argc, char **argv) {
   memset(calloc(16, 128), 48, 2048);
   while (!atomic_load(&thread_done));
 
-  __lsan::DoLeakCheck();
+  __lsan_do_leak_check();
   fprintf(stderr, "LSAN TEST COMPLETE\n");
 }
