@@ -3,11 +3,12 @@
 # University of Illinois/NCSA Open Source License.  Both these licenses can be
 # found in the LICENSE file.
 
+from .toolchain_profiler import ToolchainProfiler
+
 from subprocess import PIPE
 import atexit
 import binascii
 import base64
-import difflib
 import json
 import logging
 import os
@@ -15,6 +16,7 @@ import re
 import shutil
 import subprocess
 import time
+import signal
 import sys
 import tempfile
 
@@ -23,13 +25,14 @@ if sys.version_info < (3, 6):
   print('error: emscripten requires python 3.6 or above', file=sys.stderr)
   sys.exit(1)
 
-from .toolchain_profiler import ToolchainProfiler
 from .tempfiles import try_delete
 from .utils import path_from_root, exit_with_error, safe_ensure_dirs, WINDOWS
 from . import cache, tempfiles, colored_logger
 from . import diagnostics
 from . import config
 from . import filelock
+from . import utils
+from .settings import settings
 
 
 DEBUG = int(os.environ.get('EMCC_DEBUG', '0'))
@@ -49,7 +52,7 @@ logger = logging.getLogger('shared')
 
 # warning about absolute-paths is disabled by default, and not enabled by -Wall
 diagnostics.add_warning('absolute-paths', enabled=False, part_of_all=False)
-# unused diagnositic flags.  TODO(sbc): remove at some point
+# unused diagnostic flags.  TODO(sbc): remove at some point
 diagnostics.add_warning('almost-asm')
 diagnostics.add_warning('experimental')
 diagnostics.add_warning('invalid-input')
@@ -59,15 +62,17 @@ diagnostics.add_warning('legacy-settings', enabled=False, part_of_all=False)
 diagnostics.add_warning('linkflags')
 diagnostics.add_warning('emcc')
 diagnostics.add_warning('undefined', error=True)
-diagnostics.add_warning('deprecated')
+diagnostics.add_warning('deprecated', shared=True)
 diagnostics.add_warning('version-check')
 diagnostics.add_warning('export-main')
+diagnostics.add_warning('map-unrecognized-libraries')
 diagnostics.add_warning('unused-command-line-argument', shared=True)
 diagnostics.add_warning('pthreads-mem-growth')
 
 
 # TODO(sbc): Investigate switching to shlex.quote
 def shlex_quote(arg):
+  arg = os.fspath(arg)
   if ' ' in arg and (not (arg.startswith('"') and arg.endswith('"'))) and (not (arg.startswith("'") and arg.endswith("'"))):
     return '"' + arg.replace('"', '\\"') + '"'
 
@@ -81,7 +86,7 @@ def shlex_join(cmd):
 
 
 def run_process(cmd, check=True, input=None, *args, **kw):
-  """Runs a subpocess returning the exit code.
+  """Runs a subprocess returning the exit code.
 
   By default this function will raise an exception on failure.  Therefor this should only be
   used if you want to handle such failures.  For most subprocesses, failures are not recoverable
@@ -115,6 +120,15 @@ def mp_run_process(command_tuple):
   return ret
 
 
+def returncode_to_str(code):
+  assert code != 0
+  if code < 0:
+    signal_name = signal.Signals(-code).name
+    return f'received {signal_name} ({code})'
+
+  return f'returned {code}'
+
+
 # Runs multiple subprocess commands.
 # bool 'check': If True (default), raises an exception if any of the subprocesses failed with a nonzero exit code.
 # string 'route_stdout_to_temp_files_suffix': if not None, all stdouts are instead written to files, and an array of filenames is returned.
@@ -186,7 +200,8 @@ def run_multiple_processes(commands, env=os.environ.copy(), route_stdout_to_temp
             logger.info(out)
           if err:
             logger.error(err)
-          raise Exception('Subprocess %d/%d failed with return code %d! (cmdline: %s)' % (idx + 1, len(commands), finished_process.returncode, shlex_join(commands[idx])))
+
+          raise Exception('Subprocess %d/%d failed (%s)! (cmdline: %s)' % (idx + 1, len(commands), returncode_to_str(finished_process.returncode), shlex_join(commands[idx])))
         num_completed += 1
 
   # If processes finished out of order, sort the results to the order of the input.
@@ -200,7 +215,7 @@ def check_call(cmd, *args, **kw):
   try:
     return run_process(cmd, *args, **kw)
   except subprocess.CalledProcessError as e:
-    exit_with_error("'%s' failed (%d)", shlex_join(cmd), e.returncode)
+    exit_with_error("'%s' failed (%s)", shlex_join(cmd), returncode_to_str(e.returncode))
   except OSError as e:
     exit_with_error("'%s' failed: %s", shlex_join(cmd), str(e))
 
@@ -230,7 +245,7 @@ def timeout_run(proc, timeout=None, full_output=False, check=True):
   if check and proc.returncode != 0:
     raise subprocess.CalledProcessError(proc.returncode, '', stdout, stderr)
   if TRACK_PROCESS_SPAWNS:
-    logging.info('Process ' + str(proc.pid) + ' finished after ' + str(time.time() - start) + ' seconds. Exit code: ' + str(proc.returncode))
+    logging.info(f'Process {proc.pid} finished after {time.time() - start} seconds. Exit code: {proc.returncode}')
   return '\n'.join(out) if full_output else out[0]
 
 
@@ -240,7 +255,7 @@ def get_npm_cmd(name):
   else:
     cmd = config.NODE_JS + [path_from_root('node_modules', '.bin', name)]
   if not os.path.exists(cmd[-1]):
-    exit_with_error('%s was not found! Please run "npm install" in Emscripten root directory to set up npm dependencies' % name)
+    exit_with_error(f'{name} was not found! Please run "npm install" in Emscripten root directory to set up npm dependencies')
   return cmd
 
 
@@ -258,7 +273,7 @@ def check_llvm_version():
   actual = get_clang_version()
   if EXPECTED_LLVM_VERSION in actual:
     return True
-  diagnostics.warning('version-check', 'LLVM version appears incorrect (seeing "%s", expected "%s")', actual, EXPECTED_LLVM_VERSION)
+  diagnostics.warning('version-check', 'LLVM version for clang executable "%s" appears incorrect (seeing "%s", expected "%s")', CLANG_CC, actual, EXPECTED_LLVM_VERSION)
   return False
 
 
@@ -309,7 +324,8 @@ def check_node_version():
     return False
 
   if version < EXPECTED_NODE_VERSION:
-    diagnostics.warning('version-check', 'node version appears too old (seeing "%s", expected "%s")', actual, 'v' + ('.'.join(map(str, EXPECTED_NODE_VERSION))))
+    expected = '.'.join(str(v) for v in EXPECTED_NODE_VERSION)
+    diagnostics.warning('version-check', f'node version appears too old (seeing "{actual}", expected "v{expected}")')
     return False
 
   return True
@@ -319,14 +335,14 @@ def set_version_globals():
   global EMSCRIPTEN_VERSION, EMSCRIPTEN_VERSION_MAJOR, EMSCRIPTEN_VERSION_MINOR, EMSCRIPTEN_VERSION_TINY
   filename = path_from_root('emscripten-version.txt')
   with open(filename) as f:
-    EMSCRIPTEN_VERSION = f.read().strip().replace('"', '')
+    EMSCRIPTEN_VERSION = f.read().strip().strip('"')
   parts = [int(x) for x in EMSCRIPTEN_VERSION.split('.')]
   EMSCRIPTEN_VERSION_MAJOR, EMSCRIPTEN_VERSION_MINOR, EMSCRIPTEN_VERSION_TINY = parts
 
 
 def generate_sanity():
-  sanity_file_content = EMSCRIPTEN_VERSION + '|' + config.LLVM_ROOT + '|' + get_clang_version()
-  config_data = open(config.EM_CONFIG).read()
+  sanity_file_content = f'{EMSCRIPTEN_VERSION}|{config.LLVM_ROOT}|{get_clang_version()}'
+  config_data = utils.read_file(config.EM_CONFIG)
   checksum = binascii.crc32(config_data.encode())
   sanity_file_content += '|%#x\n' % checksum
   return sanity_file_content
@@ -360,6 +376,7 @@ def perform_sanity_checks():
         exit_with_error('Cannot find %s, check the paths in %s', cmd, config.EM_CONFIG)
 
 
+@ToolchainProfiler.profile_block('sanity')
 def check_sanity(force=False):
   """Check that basic stuff we need (a JS engine to compile, Node.js, and Clang
   and LLVM) exists.
@@ -388,36 +405,35 @@ def check_sanity(force=False):
     perform_sanity_checks()
     return
 
-  with ToolchainProfiler.profile_block('sanity'):
-    expected = generate_sanity()
+  expected = generate_sanity()
 
-    sanity_file = Cache.get_path('sanity.txt')
-    with Cache.lock():
-      if os.path.exists(sanity_file):
-        sanity_data = open(sanity_file).read()
-        if sanity_data != expected:
-          logger.debug('old sanity: %s' % sanity_data)
-          logger.debug('new sanity: %s' % expected)
-          logger.info('(Emscripten: config changed, clearing cache)')
-          Cache.erase()
-          # the check actually failed, so definitely write out the sanity file, to
-          # avoid others later seeing failures too
-          force = False
-        else:
-          if force:
-            logger.debug(f'sanity file up-to-date but check forced: {sanity_file}')
-          else:
-            logger.debug(f'sanity file up-to-date: {sanity_file}')
-            return # all is well
+  sanity_file = Cache.get_path('sanity.txt')
+  with Cache.lock():
+    if os.path.exists(sanity_file):
+      sanity_data = utils.read_file(sanity_file)
+      if sanity_data != expected:
+        logger.debug('old sanity: %s' % sanity_data)
+        logger.debug('new sanity: %s' % expected)
+        logger.info('(Emscripten: config changed, clearing cache)')
+        Cache.erase()
+        # the check actually failed, so definitely write out the sanity file, to
+        # avoid others later seeing failures too
+        force = False
       else:
-        logger.debug(f'sanity file not found: {sanity_file}')
+        if force:
+          logger.debug(f'sanity file up-to-date but check forced: {sanity_file}')
+        else:
+          logger.debug(f'sanity file up-to-date: {sanity_file}')
+          return # all is well
+    else:
+      logger.debug(f'sanity file not found: {sanity_file}')
 
-      perform_sanity_checks()
+    perform_sanity_checks()
 
-      if not force:
-        # Only create/update this file if the sanity check succeeded, i.e., we got here
-        with open(sanity_file, 'w') as f:
-          f.write(expected)
+    if not force:
+      # Only create/update this file if the sanity check succeeded, i.e., we got here
+      with open(sanity_file, 'w') as f:
+        f.write(expected)
 
 
 # Some distributions ship with multiple llvm versions so they add
@@ -456,7 +472,7 @@ def replace_suffix(filename, new_suffix):
 # Retain the original naming scheme in traditional runtime.
 def replace_or_append_suffix(filename, new_suffix):
   assert new_suffix[0] == '.'
-  return replace_suffix(filename, new_suffix) if Settings.MINIMAL_RUNTIME else filename + new_suffix
+  return replace_suffix(filename, new_suffix) if settings.MINIMAL_RUNTIME else filename + new_suffix
 
 
 # Temp dir. Create a random one, unless EMCC_DEBUG is set, in which case use the canonical
@@ -488,7 +504,7 @@ class Configuration:
 
     self.TEMP_DIR = os.environ.get("EMCC_TEMP_DIR", tempfile.gettempdir())
     if not os.path.isdir(self.TEMP_DIR):
-      exit_with_error("The temporary directory `" + self.TEMP_DIR + "` does not exist! Please make sure that the path is correct.")
+      exit_with_error(f'The temporary directory `{self.TEMP_DIR}` does not exist! Please make sure that the path is correct.')
 
     self.CANONICAL_TEMP_DIR = get_canonical_temp_dir(self.TEMP_DIR)
 
@@ -497,7 +513,7 @@ class Configuration:
       try:
         safe_ensure_dirs(self.EMSCRIPTEN_TEMP_DIR)
       except Exception as e:
-        exit_with_error(str(e) + 'Could not create canonical temp dir. Check definition of TEMP_DIR in ' + config.EM_CONFIG)
+        exit_with_error(str(e) + f'Could not create canonical temp dir. Check definition of TEMP_DIR in {config.EM_CONFIG}')
 
       # Since the canonical temp directory is, by definition, the same
       # between all processes that run in DEBUG mode we need to use a multi
@@ -532,170 +548,8 @@ def apply_configuration():
   TEMP_DIR = configuration.TEMP_DIR
 
 
-# Settings. A global singleton. Not pretty, but nicer than passing |, settings| everywhere
-class SettingsManager:
-
-  class __impl:
-    attrs = {}
-    internal_settings = set()
-
-    def __init__(self):
-      self.reset()
-
-    @classmethod
-    def reset(cls):
-      cls.attrs = {}
-
-      # Load the JS defaults into python.
-      settings = open(path_from_root('src', 'settings.js')).read().replace('//', '#')
-      settings = re.sub(r'var ([\w\d]+)', r'attrs["\1"]', settings)
-      # Variable TARGET_NOT_SUPPORTED is referenced by value settings.js (also beyond declaring it),
-      # so must pass it there explicitly.
-      exec(settings, {'attrs': cls.attrs})
-
-      settings = open(path_from_root('src', 'settings_internal.js')).read().replace('//', '#')
-      settings = re.sub(r'var ([\w\d]+)', r'attrs["\1"]', settings)
-      internal_attrs = {}
-      exec(settings, {'attrs': internal_attrs})
-      cls.attrs.update(internal_attrs)
-
-      if 'EMCC_STRICT' in os.environ:
-        cls.attrs['STRICT'] = int(os.environ.get('EMCC_STRICT'))
-
-      # Special handling for LEGACY_SETTINGS.  See src/setting.js for more
-      # details
-      cls.legacy_settings = {}
-      cls.alt_names = {}
-      for legacy in cls.attrs['LEGACY_SETTINGS']:
-        if len(legacy) == 2:
-          name, new_name = legacy
-          cls.legacy_settings[name] = (None, 'setting renamed to ' + new_name)
-          cls.alt_names[name] = new_name
-          cls.alt_names[new_name] = name
-          default_value = cls.attrs[new_name]
-        else:
-          name, fixed_values, err = legacy
-          cls.legacy_settings[name] = (fixed_values, err)
-          default_value = fixed_values[0]
-        assert name not in cls.attrs, 'legacy setting (%s) cannot also be a regular setting' % name
-        if not cls.attrs['STRICT']:
-          cls.attrs[name] = default_value
-
-      cls.internal_settings = set(internal_attrs.keys())
-
-    # Transforms the Settings information into emcc-compatible args (-s X=Y, etc.). Basically
-    # the reverse of load_settings, except for -Ox which is relevant there but not here
-    @classmethod
-    def serialize(cls):
-      ret = []
-      for key, value in cls.attrs.items():
-        if key == key.upper():  # this is a hack. all of our settings are ALL_CAPS, python internals are not
-          jsoned = json.dumps(value, sort_keys=True)
-          ret += ['-s', key + '=' + jsoned]
-      return ret
-
-    @classmethod
-    def to_dict(cls):
-      return cls.attrs.copy()
-
-    @classmethod
-    def copy(cls, values):
-      cls.attrs = values
-
-    @classmethod
-    def apply_opt_level(cls, opt_level, shrink_level=0, noisy=False):
-      if opt_level >= 1:
-        cls.attrs['ASSERTIONS'] = 0
-      if shrink_level >= 2:
-        cls.attrs['EVAL_CTORS'] = 1
-
-    def keys(self):
-      return self.attrs.keys()
-
-    def __getattr__(self, attr):
-      if attr in self.attrs:
-        return self.attrs[attr]
-      else:
-        raise AttributeError("Settings object has no attribute '%s'" % attr)
-
-    def __setattr__(self, attr, value):
-      if attr == 'STRICT' and value:
-        for a in self.legacy_settings:
-          self.attrs.pop(a, None)
-
-      if attr in self.legacy_settings:
-        # TODO(sbc): Rather then special case this we should have STRICT turn on the
-        # legacy-settings warning below
-        if self.attrs['STRICT']:
-          exit_with_error('legacy setting used in strict mode: %s', attr)
-        fixed_values, error_message = self.legacy_settings[attr]
-        if fixed_values and value not in fixed_values:
-          exit_with_error('Invalid command line option -s ' + attr + '=' + str(value) + ': ' + error_message)
-        diagnostics.warning('legacy-settings', 'use of legacy setting: %s (%s)', attr, error_message)
-
-      if attr in self.alt_names:
-        alt_name = self.alt_names[attr]
-        self.attrs[alt_name] = value
-
-      if attr not in self.attrs:
-        msg = "Attempt to set a non-existent setting: '%s'\n" % attr
-        suggestions = difflib.get_close_matches(attr, list(self.attrs.keys()))
-        suggestions = [s for s in suggestions if s not in self.legacy_settings]
-        suggestions = ', '.join(suggestions)
-        if suggestions:
-          msg += ' - did you mean one of %s?\n' % suggestions
-        msg += " - perhaps a typo in emcc's  -s X=Y  notation?\n"
-        msg += ' - (see src/settings.js for valid values)'
-        exit_with_error(msg)
-
-      self.attrs[attr] = value
-
-    @classmethod
-    def get(cls, key):
-      return cls.attrs.get(key)
-
-    @classmethod
-    def __getitem__(cls, key):
-      return cls.attrs[key]
-
-    @classmethod
-    def target_environment_may_be(self, environment):
-      return self.attrs['ENVIRONMENT'] == '' or environment in self.attrs['ENVIRONMENT'].split(',')
-
-  __instance = None
-
-  @staticmethod
-  def instance():
-    if SettingsManager.__instance is None:
-      SettingsManager.__instance = SettingsManager.__impl()
-    return SettingsManager.__instance
-
-  def __getattr__(self, attr):
-    return getattr(self.instance(), attr)
-
-  def __setattr__(self, attr, value):
-    return setattr(self.instance(), attr, value)
-
-  def get(self, key):
-    return self.instance().get(key)
-
-  def __getitem__(self, key):
-    return self.instance()[key]
-
-
-def verify_settings():
-  if Settings.SAFE_HEAP not in [0, 1]:
-    exit_with_error('emcc: SAFE_HEAP must be 0 or 1 in fastcomp')
-
-  if not Settings.WASM:
-    # When the user requests non-wasm output, we enable wasm2js. that is,
-    # we still compile to wasm normally, but we compile the final output
-    # to js.
-    Settings.WASM = 1
-    Settings.WASM2JS = 1
-  if Settings.WASM == 2:
-    # Requesting both Wasm and Wasm2JS support
-    Settings.WASM2JS = 1
+def target_environment_may_be(environment):
+  return not settings.ENVIRONMENT or environment in settings.ENVIRONMENT.split(',')
 
 
 def print_compiler_stage(cmd):
@@ -721,7 +575,7 @@ def is_c_symbol(name):
 def treat_as_user_function(name):
   if name.startswith('dynCall_'):
     return False
-  if name in Settings.WASM_SYSTEM_EXPORTS:
+  if name in settings.WASM_SYSTEM_EXPORTS:
     return False
   return True
 
@@ -738,6 +592,11 @@ def asmjs_mangle(name):
     return name
 
 
+def reconfigure_cache():
+  global Cache
+  Cache = cache.Cache(config.CACHE)
+
+
 class JS:
   emscripten_license = '''\
 /**
@@ -765,7 +624,7 @@ class JS:
       js = f.read()
     # first, remove the license as there may be more than once
     processed_js = re.sub(JS.emscripten_license_regex, '', js)
-    if Settings.EMIT_EMSCRIPTEN_LICENSE:
+    if settings.EMIT_EMSCRIPTEN_LICENSE:
       processed_js = JS.emscripten_license + processed_js
     if processed_js != js:
       with open(js_target, 'w') as f:
@@ -785,7 +644,7 @@ class JS:
   @staticmethod
   def get_subresource_location(path, data_uri=None):
     if data_uri is None:
-      data_uri = Settings.SINGLE_FILE
+      data_uri = settings.SINGLE_FILE
     if data_uri:
       # if the path does not exist, then there is no data to encode
       if not os.path.exists(path):
@@ -799,7 +658,7 @@ class JS:
   @staticmethod
   def legalize_sig(sig):
     # with BigInt support all sigs are legal since we can use i64s.
-    if Settings.WASM_BIGINT:
+    if settings.WASM_BIGINT:
       return sig
     legal = [sig[0]]
     # a return of i64 is legalized into an i32 (and the high bits are
@@ -818,16 +677,21 @@ class JS:
   @staticmethod
   def is_legal_sig(sig):
     # with BigInt support all sigs are legal since we can use i64s.
-    if Settings.WASM_BIGINT:
+    if settings.WASM_BIGINT:
       return True
     return sig == JS.legalize_sig(sig)
 
+  @staticmethod
+  def isidentifier(name):
+    # https://stackoverflow.com/questions/43244604/check-that-a-string-is-a-valid-javascript-identifier-name-using-python-3
+    return name.replace('$', '_').isidentifier()
+
   @staticmethod
   def make_dynCall(sig, args):
     # wasm2c and asyncify are not yet compatible with direct wasm table calls
-    if Settings.DYNCALLS or not JS.is_legal_sig(sig):
+    if settings.DYNCALLS or not JS.is_legal_sig(sig):
       args = ','.join(args)
-      if not Settings.MAIN_MODULE and not Settings.SIDE_MODULE:
+      if not settings.MAIN_MODULE and not settings.SIDE_MODULE:
         # Optimize dynCall accesses in the case when not building with dynamic
         # linking enabled.
         return 'dynCall_%s(%s)' % (sig, args)
@@ -843,7 +707,7 @@ class JS:
     ret = 'return ' if sig[0] != 'v' else ''
     body = '%s%s;' % (ret, JS.make_dynCall(sig, args))
     # C++ exceptions are numbers, and longjmp is a string 'longjmp'
-    if Settings.SUPPORT_LONGJMP:
+    if settings.SUPPORT_LONGJMP:
       rethrow = "if (e !== e+0 && e !== 'longjmp') throw e;"
     else:
       rethrow = "if (e !== e+0) throw e;"
@@ -870,7 +734,7 @@ def suffix(name):
 
 
 def unsuffixed(name):
-  """Return the filename without the extention.
+  """Return the filename without the extension.
 
   If there are multiple extensions this strips only the final one.
   """
@@ -881,6 +745,11 @@ def unsuffixed_basename(name):
   return os.path.basename(unsuffixed(name))
 
 
+def strip_prefix(string, prefix):
+  assert string.startswith(prefix)
+  return string[len(prefix):]
+
+
 def safe_copy(src, dst):
   logging.debug('copy: %s -> %s', src, dst)
   src = os.path.abspath(src)
@@ -891,15 +760,20 @@ def safe_copy(src, dst):
     return
   if dst == os.devnull:
     return
-  shutil.copyfile(src, dst)
+  # Copies data and permission bits, but not other metadata such as timestamp
+  shutil.copy(src, dst)
 
 
 def read_and_preprocess(filename, expand_macros=False):
   temp_dir = get_emscripten_temp_dir()
   # Create a settings file with the current settings to pass to the JS preprocessor
-  # Note: Settings.serialize returns an array of -s options i.e. ['-s', '<setting1>', '-s', '<setting2>', ...]
-  #       we only want the actual settings, hence the [1::2] slice operation.
-  settings_str = "var " + ";\nvar ".join(Settings.serialize()[1::2])
+
+  settings_str = ''
+  for key, value in settings.dict().items():
+    assert key == key.upper()  # should only ever be uppercase keys in settings
+    jsoned = json.dumps(value, sort_keys=True)
+    settings_str += f'var {key} = {jsoned};\n'
+
   settings_file = os.path.join(temp_dir, 'settings.js')
   with open(settings_file, 'w') as f:
     f.write(settings_str)
@@ -917,7 +791,7 @@ def read_and_preprocess(filename, expand_macros=False):
     args += ['--expandMacros']
 
   run_js_tool(path_from_root('tools/preprocessor.js'), args, True, stdout=open(stdout, 'w'), cwd=dirname)
-  out = open(stdout, 'r').read()
+  out = utils.read_file(stdout)
 
   return out
 
@@ -950,6 +824,7 @@ CLANG_CC = os.path.expanduser(build_clang_tool_path(exe_suffix('clang')))
 CLANG_CXX = os.path.expanduser(build_clang_tool_path(exe_suffix('clang++')))
 LLVM_LINK = build_llvm_tool_path(exe_suffix('llvm-link'))
 LLVM_AR = build_llvm_tool_path(exe_suffix('llvm-ar'))
+LLVM_DWP = build_llvm_tool_path(exe_suffix('llvm-dwp'))
 LLVM_RANLIB = build_llvm_tool_path(exe_suffix('llvm-ranlib'))
 LLVM_OPT = os.path.expanduser(build_llvm_tool_path(exe_suffix('opt')))
 LLVM_NM = os.path.expanduser(build_llvm_tool_path(exe_suffix('llvm-nm')))
@@ -965,12 +840,11 @@ EMAR = bat_suffix(path_from_root('emar'))
 EMRANLIB = bat_suffix(path_from_root('emranlib'))
 EMCMAKE = bat_suffix(path_from_root('emcmake'))
 EMCONFIGURE = bat_suffix(path_from_root('emconfigure'))
+EM_NM = bat_suffix(path_from_root('emnm'))
 FILE_PACKAGER = bat_suffix(path_from_root('tools', 'file_packager'))
 
 apply_configuration()
 
-Settings = SettingsManager()
-verify_settings()
 Cache = cache.Cache(config.CACHE)
 
 PRINT_STAGES = int(os.getenv('EMCC_VERBOSE', '0'))
