@@ -98,10 +98,9 @@ def create_lib(libname, inputs):
         shutil.copyfile(inputs[0], libname)
     else:
       building.link_to_object(inputs, libname)
-  elif suffix == '.a':
-    building.emar('cr', libname, inputs)
   else:
-    raise Exception('unknown suffix ' + libname)
+    assert suffix == '.a'
+    building.emar('cr', libname, inputs)
 
 
 def get_wasm_libc_rt_files():
@@ -151,6 +150,14 @@ def get_wasm_libc_rt_files():
   return math_files + other_files + iprintf_files
 
 
+def is_case_insensitive(path):
+  """Returns True if the filesystem at `path` is case insensitive."""
+  utils.write_file(os.path.join(path, 'test_file'), '')
+  case_insensitive = os.path.exists(os.path.join(path, 'TEST_FILE'))
+  os.remove(os.path.join(path, 'test_file'))
+  return case_insensitive
+
+
 class Library:
   """
   `Library` is the base class of all system libraries.
@@ -343,8 +350,20 @@ class Library:
     objects = []
     cflags = self.get_cflags()
     base_flags = get_base_cflags()
+    case_insensitive = is_case_insensitive(build_dir)
     for src in self.get_files():
-      o = os.path.join(build_dir, shared.unsuffixed_basename(src) + '.o')
+      object_basename = shared.unsuffixed_basename(src)
+      # Resolve duplicates by appending unique.
+      # This is needed on case insensitve filesystem to handle,
+      # for example, _exit.o and _Exit.o.
+      if case_insensitive:
+        object_basename = object_basename.lower()
+      o = os.path.join(build_dir, object_basename + '.o')
+      object_uuid = 0
+      # Find a unique basename
+      while o in objects:
+        object_uuid += 1
+        o = os.path.join(build_dir, f'{object_basename}__{object_uuid}.o')
       ext = shared.suffix(src)
       if ext in ('.s', '.S', '.c'):
         cmd = [shared.EMCC]
@@ -693,7 +712,7 @@ class libc(AsanInstrumentedLibrary, MuslInternalLibrary, MTLibrary):
 
     # musl modules
     ignore = [
-        'ipc', 'passwd', 'signal', 'sched', 'ipc', 'time', 'linux',
+        'ipc', 'passwd', 'signal', 'sched', 'time', 'linux',
         'aio', 'exit', 'legacy', 'mq', 'setjmp', 'env',
         'ldso'
     ]
@@ -704,7 +723,7 @@ class libc(AsanInstrumentedLibrary, MuslInternalLibrary, MTLibrary):
         'res_query.c', 'res_querydomain.c', 'gai_strerror.c',
         'proto.c', 'gethostbyaddr.c', 'gethostbyaddr_r.c', 'gethostbyname.c',
         'gethostbyname2_r.c', 'gethostbyname_r.c', 'gethostbyname2.c',
-        'alarm.c', 'syscall.c', '_exit.c', 'popen.c',
+        'alarm.c', 'syscall.c', 'popen.c',
         'getgrouplist.c', 'initgroups.c', 'wordexp.c', 'timer_create.c',
         'faccessat.c',
         # 'process' exclusion
@@ -713,25 +732,10 @@ class libc(AsanInstrumentedLibrary, MuslInternalLibrary, MTLibrary):
 
     ignore += LIBC_SOCKETS
 
-    if self.is_asan:
-      # With ASan, we need to use specialized implementations of certain libc
-      # functions that do not rely on undefined behavior, for example, reading
-      # multiple bytes at once as an int and overflowing a buffer.
-      # Otherwise, ASan will catch these errors and terminate the program.
-      ignore += ['strcpy.c', 'memchr.c', 'strchrnul.c', 'strlen.c',
-                 'aligned_alloc.c', 'fcntl.c']
-      libc_files += [
-        shared.path_from_root('system', 'lib', 'libc', 'emscripten_asan_strcpy.c'),
-        shared.path_from_root('system', 'lib', 'libc', 'emscripten_asan_memchr.c'),
-        shared.path_from_root('system', 'lib', 'libc', 'emscripten_asan_strchrnul.c'),
-        shared.path_from_root('system', 'lib', 'libc', 'emscripten_asan_strlen.c'),
-        shared.path_from_root('system', 'lib', 'libc', 'emscripten_asan_fcntl.c'),
-      ]
-
     if self.is_mt:
       ignore += [
         'clone.c', '__lock.c',
-        'pthread_cleanup_push.c', 'pthread_create.c',
+        'pthread_create.c',
         'pthread_kill.c', 'pthread_sigmask.c',
         '__set_thread_area.c', 'synccall.c',
         '__syscall_cp.c', '__tls_get_addr.c',
@@ -743,7 +747,7 @@ class libc(AsanInstrumentedLibrary, MuslInternalLibrary, MTLibrary):
         # TODO: No longer exists in the latest musl version.
         '__futex.c',
         # TODO: Could be supported in the upcoming musl upgrade
-        'lock_ptc.c', 'pthread_getattr_np.c',
+        'lock_ptc.c',
         # 'pthread_setattr_default_np.c',
         # TODO: These could be moved away from JS in the upcoming musl upgrade.
         'pthread_cancel.c', 'pthread_detach.c',
@@ -761,6 +765,7 @@ class libc(AsanInstrumentedLibrary, MuslInternalLibrary, MTLibrary):
         path_components=['system', 'lib', 'libc', 'musl', 'src', 'thread'],
         filenames=[
           'pthread_self.c',
+          'pthread_cleanup_push.c',
           # C11 thread library functions
           'call_once.c',
           'tss_create.c',
@@ -825,12 +830,33 @@ class libc(AsanInstrumentedLibrary, MuslInternalLibrary, MTLibrary):
         path_components=['system', 'lib', 'libc', 'musl', 'src', 'sched'],
         filenames=['sched_yield.c'])
 
+    libc_files += files_in_path(
+        path_components=['system', 'lib', 'libc', 'musl', 'src', 'exit'],
+        filenames=['_Exit.c'])
+
+    libc_files += files_in_path(
+        path_components=['system', 'lib', 'libc', 'musl', 'src', 'signal'],
+        filenames=[
+          'getitimer.c',
+          'killpg.c',
+          'setitimer.c',
+          'sigaddset.c',
+          'sigdelset.c',
+          'sigemptyset.c',
+          'sigfillset.c',
+          'sigismember.c',
+          'signal.c',
+          'sigprocmask.c',
+          'sigrtmax.c',
+          'sigrtmin.c',
+          'sigwait.c',
+        ])
+
     libc_files += files_in_path(
         path_components=['system', 'lib', 'libc'],
         filenames=[
           'extras.c',
           'wasi-helpers.c',
-          'emscripten_pthread.c',
           'emscripten_get_heap_size.c',
         ])
 
@@ -932,6 +958,9 @@ class libcxxabi(NoExceptLibrary, MTLibrary):
       cflags.append('-D_LIBCXXABI_NO_EXCEPTIONS')
     elif self.eh_mode == Exceptions.EMSCRIPTEN:
       cflags.append('-D__USING_EMSCRIPTEN_EXCEPTIONS__')
+      # The code used to interpret exceptions during terminate
+      # is not compatible with emscripten exceptions.
+      cflags.append('-DLIBCXXABI_SILENT_TERMINATE')
     elif self.eh_mode == Exceptions.WASM:
       cflags.append('-D__USING_WASM_EXCEPTIONS__')
     return cflags
@@ -942,7 +971,6 @@ class libcxxabi(NoExceptLibrary, MTLibrary):
       'cxa_aux_runtime.cpp',
       'cxa_default_handlers.cpp',
       'cxa_demangle.cpp',
-      'cxa_exception_storage.cpp',
       'cxa_guard.cpp',
       'cxa_handlers.cpp',
       'cxa_virtual.cpp',
@@ -957,6 +985,7 @@ class libcxxabi(NoExceptLibrary, MTLibrary):
       filenames += ['cxa_noexception.cpp']
     elif self.eh_mode == Exceptions.WASM:
       filenames += [
+        'cxa_exception_storage.cpp',
         'cxa_exception.cpp',
         'cxa_personality.cpp'
       ]
@@ -1329,7 +1358,7 @@ class libstandalonewasm(MuslInternalLibrary):
   # LTO defeats the weak linking trick used in __original_main.c
   force_object_files = True
 
-  cflags = ['-Os']
+  cflags = ['-Os', '-fno-builtin']
   src_dir = ['system', 'lib']
 
   def __init__(self, **kwargs):
@@ -1344,9 +1373,9 @@ class libstandalonewasm(MuslInternalLibrary):
 
   def get_cflags(self):
     cflags = super().get_cflags()
-    cflags += ['-DNDEBUG']
+    cflags += ['-DNDEBUG', '-DEMSCRIPTEN_STANDALONE_WASM']
     if self.is_mem_grow:
-      cflags += ['-D__EMSCRIPTEN_MEMORY_GROWTH__=1']
+      cflags += ['-DEMSCRIPTEN_MEMORY_GROWTH']
     return cflags
 
   @classmethod
@@ -1361,12 +1390,15 @@ class libstandalonewasm(MuslInternalLibrary):
     )
 
   def get_files(self):
-    base_files = files_in_path(
+    files = files_in_path(
         path_components=['system', 'lib', 'standalone'],
         filenames=['standalone.c', 'standalone_wasm_stdio.c', '__original_main.c',
                    '__main_void.c', '__main_argc_argv.c'])
+    files += files_in_path(
+        path_components=['system', 'lib', 'libc'],
+        filenames=['emscripten_memcpy.c'])
     # It is more efficient to use JS methods for time, normally.
-    time_files = files_in_path(
+    files += files_in_path(
         path_components=['system', 'lib', 'libc', 'musl', 'src', 'time'],
         filenames=['strftime.c',
                    '__month_to_secs.c',
@@ -1385,12 +1417,10 @@ class libstandalonewasm(MuslInternalLibrary):
                    'time.c'])
     # It is more efficient to use JS for __assert_fail, as it avoids always
     # including fprintf etc.
-    exit_files = files_in_path(
+    files += files_in_path(
         path_components=['system', 'lib', 'libc', 'musl', 'src', 'exit'],
-        filenames=['assert.c', 'atexit.c', 'exit.c']) + files_in_path(
-        path_components=['system', 'lib', 'libc', 'musl', 'src', 'unistd'],
-        filenames=['_exit.c'])
-    return base_files + time_files + exit_files
+        filenames=['assert.c', 'atexit.c', 'exit.c'])
+    return files
 
 
 class libjsmath(Library):
@@ -1491,7 +1521,7 @@ def calculate(input_files, forced):
     force = ','.join(name for name, lib in system_libs_map.items() if not lib.never_force)
   force_include = set((force.split(',') if force else []) + forced)
   if force_include:
-    logger.debug('forcing stdlibs: ' + str(force_include))
+    logger.debug(f'forcing stdlibs: {force_include}')
 
   def add_library(libname):
     lib = system_libs_map[libname]
@@ -1631,18 +1661,18 @@ class Ports:
       target = os.path.basename(src_dir)
     dest = os.path.join(Ports.get_include_dir(), target)
     shared.try_delete(dest)
-    logger.debug('installing headers: ' + dest)
+    logger.debug(f'installing headers: {dest}')
     shutil.copytree(src_dir, dest)
 
   @staticmethod
-  def install_headers(src_dir, pattern="*.h", target=None):
-    logger.debug("install_headers")
+  def install_headers(src_dir, pattern='*.h', target=None):
+    logger.debug('install_headers')
     dest = Ports.get_include_dir()
     if target:
       dest = os.path.join(dest, target)
       shared.safe_ensure_dirs(dest)
     matches = glob.glob(os.path.join(src_dir, pattern))
-    assert matches, "no headers found to install in %s" % src_dir
+    assert matches, f'no headers found to install in {src_dir}'
     for f in matches:
       logger.debug('installing: ' + os.path.join(dest, os.path.basename(f)))
       shutil.copyfile(f, os.path.join(dest, os.path.basename(f)))
@@ -1736,34 +1766,31 @@ class Ports:
               shared.exit_with_error('%s is not a known port' % name)
             port = ports.ports_by_name[name]
             if not hasattr(port, 'SUBDIR'):
-              logger.error('port %s lacks .SUBDIR attribute, which we need in order to override it locally, please update it' % name)
+              logger.error(f'port {name} lacks .SUBDIR attribute, which we need in order to override it locally, please update it')
               sys.exit(1)
             subdir = port.SUBDIR
             target = os.path.join(fullname, subdir)
             if os.path.exists(target) and not dir_is_newer(path, target):
-              logger.warning('not grabbing local port: ' + name + ' from ' + path + ' to ' + fullname + ' (subdir: ' + subdir + ') as the destination ' + target + ' is newer (run emcc --clear-ports if that is incorrect)')
+              logger.warning(f'not grabbing local port: {name} from {path} to {fullname} (subdir: {subdir}) as the destination {target} is newer (run emcc --clear-ports if that is incorrect)')
             else:
-              logger.warning('grabbing local port: ' + name + ' from ' + path + ' to ' + fullname + ' (subdir: ' + subdir + ')')
+              logger.warning(f'grabbing local port: {name} from {path} to {fullname} (subdir: {subdir})')
               shared.try_delete(fullname)
               shutil.copytree(path, target)
               Ports.clear_project_build(name)
             return
 
-    if url.endswith('.tar.bz2'):
-      fullpath = fullname + '.tar.bz2'
-    elif url.endswith('.tar.gz'):
-      fullpath = fullname + '.tar.gz'
-    else:
-      fullpath = fullname + '.zip'
+    url_filename = url.rsplit('/')[-1]
+    ext = url_filename.split('.', 1)[1]
+    fullpath = fullname + '.' + ext
 
     if name not in Ports.name_cache: # only mention each port once in log
-      logger.debug('including port: ' + name)
-      logger.debug('    (at ' + fullname + ')')
+      logger.debug(f'including port: {name}')
+      logger.debug(f'    (at {fullname})')
       Ports.name_cache.add(name)
 
     def retrieve():
       # retrieve from remote server
-      logger.info('retrieving port: ' + name + ' from ' + url)
+      logger.info(f'retrieving port: {name} from {url}')
       try:
         import requests
         response = requests.get(url)
@@ -1776,7 +1803,7 @@ class Ports:
       if sha512hash:
         actual_hash = hashlib.sha512(data).hexdigest()
         if actual_hash != sha512hash:
-          shared.exit_with_error('Unexpected hash: ' + actual_hash + '\n'
+          shared.exit_with_error(f'Unexpected hash: {actual_hash}\n'
                                  'If you are updating the port, please update the hash in the port module.')
       with open(fullpath, 'wb') as f:
         f.write(data)
@@ -1784,7 +1811,7 @@ class Ports:
     marker = os.path.join(fullname, '.emscripten_url')
 
     def unpack():
-      logger.info('unpacking port: ' + name)
+      logger.info(f'unpacking port: {name}')
       shared.safe_ensure_dirs(fullname)
       shutil.unpack_archive(filename=fullpath, extract_dir=fullname)
       with open(marker, 'w') as f:
